"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const root = path_1.default.resolve(__dirname, '../client-templates/');
const logger_1 = require("../lib/logs/logger");
exports.default = async (templateName) => {
    if (!templateName) {
        throw new Error('init requires a template name');
    }
    const templateDir = path_1.default.resolve(root, templateName);
    const rootFiles = await fs_1.promises.readdir(root);
    if (!rootFiles.includes(templateName)) {
        throw new Error(`${templateName} is an invalid template name`);
    }
    const templateFiles = await fs_1.promises.readdir(templateDir);
    // check if any of the files exist on disk already, and if so, abort.
    const filesWithAccessData = await Promise.all(templateFiles.map(async (file) => {
        file = file.replace(/\.sample$/, '');
        try {
            await fs_1.promises.access(file);
            return { file, exists: true };
        }
        catch (error) {
            return { file, exists: false };
        }
    }));
    const existingFiles = filesWithAccessData.filter(({ exists }) => exists);
    if (existingFiles.length) {
        const existingFileNames = existingFiles.map(({ file }) => file).join(' ');
        throw new Error(`The following file(s) exist and cannot be overwritten, please move them: ${existingFileNames}`);
    }
    await Promise.all(templateFiles.map((file) => new Promise((resolve, reject) => {
        const newfile = file.replace(/\.sample$/, '');
        logger_1.log.debug({ templateFile: newfile }, 'generating template file');
        const reader = (0, fs_1.createReadStream)(path_1.default.resolve(templateDir, file));
        const writer = (0, fs_1.createWriteStream)(path_1.default.resolve(process.cwd(), newfile));
        reader.pipe(writer).on('finish', resolve).on('error', reject);
    })));
    logger_1.log.info({ templateName }, 'initialisation complete');
};
//# sourceMappingURL=init.js.map