"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleCheckIdsRoutes = exports.handleChecksRoute = void 0;
const logger_1 = require("../../../logs/logger");
const http_1 = require("../http");
const handleChecksRoute = (config) => {
    return async (_, res) => {
        try {
            const checks = (0, http_1.getHttpChecks)(config).filter((c) => c.enabled);
            const checkResults = [];
            const results = await Promise.allSettled(checks.map(async (c) => c.check(config)));
            results.forEach((r) => {
                if (r.status === 'fulfilled') {
                    checkResults.push(r.value);
                }
                else {
                    logger_1.log.error({ error: r.reason }, 'Unexpected error when executing checks');
                }
            });
            res.status(200).json(checkResults);
        }
        catch (error) {
            logger_1.log.error({ error }, 'Error executing http checks');
            res.status(500).json({ error: error });
        }
    };
};
exports.handleChecksRoute = handleChecksRoute;
const handleCheckIdsRoutes = (config) => {
    return async (req, res) => {
        try {
            const check = (0, http_1.getHttpChecks)(config)
                .filter((c) => c.enabled)
                .find((c) => c.id === req.params.checkId);
            if (check === undefined) {
                res.sendStatus(404);
                return;
            }
            const checkResult = await check.check(config);
            res.status(200).json(checkResult);
        }
        catch (error) {
            logger_1.log.error({ error }, `Error executing check with id: ${req.params.checkId}`);
            res.status(500).json({ error: error });
        }
    };
};
exports.handleCheckIdsRoutes = handleCheckIdsRoutes;
//# sourceMappingURL=checks-handler.js.map