"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHttpChecks = void 0;
const http_executor_1 = require("./http-executor");
const dispatcher_1 = require("../../dispatcher");
const defaultTimeoutMs = 10000;
function getHttpChecks(config) {
    return [brokerServerStatusCheck(config), restApiStatusCheck(config)];
}
exports.getHttpChecks = getHttpChecks;
const brokerServerStatusCheck = (config) => {
    const url = config.BROKER_SERVER_URL || 'https://broker.snyk.io';
    return {
        id: 'broker-server-status',
        name: 'Broker Server Healthcheck',
        enabled: true,
        url: `${url}/healthcheck`,
        method: 'GET',
        timeoutMs: defaultTimeoutMs,
        check: async function () {
            return await (0, http_executor_1.executeHttpRequest)({ id: this.id, name: this.name }, { url: this.url, method: this.method, timeoutMs: this.timeoutMs });
        },
    };
};
const restApiStatusCheck = (config) => {
    const url = config.API_BASE_URL || config.BROKER_SERVER_URL
        ? config.BROKER_SERVER_URL.replace('//broker.', '//api.')
        : 'https://api.snyk.io';
    const enabled = (0, dispatcher_1.highAvailabilityModeEnabled)(config);
    return {
        id: 'rest-api-status',
        name: 'REST API Healthcheck',
        enabled: enabled,
        url: `${url}/rest/openapi`,
        method: 'GET',
        timeoutMs: defaultTimeoutMs,
        check: async function () {
            return await (0, http_executor_1.executeHttpRequest)({ id: this.id, name: this.name }, { url: this.url, method: this.method, timeoutMs: this.timeoutMs });
        },
    };
};
//# sourceMappingURL=index.js.map