"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpDispatcherServiceClient = void 0;
const request_1 = require("../../../common/http/request");
const logger_1 = require("../../../logs/logger");
class HttpDispatcherServiceClient {
    constructor(baseUrl) {
        this.version = '2022-12-01~experimental';
        this.baseUrl = baseUrl;
    }
    async createConnection(params, data) {
        try {
            const path = `/hidden/broker/${params.hashedBrokerToken}/connections/${params.brokerClientId}`;
            const url = new URL(path, this.baseUrl);
            url.searchParams.append('version', this.version);
            const req = {
                url: url.toString(),
                method: 'POST',
                headers: {
                    'Content-type': 'application/vnd.api+json',
                },
                body: JSON.stringify({
                    data: {
                        attributes: {
                            deployment_location: data.deployment_location,
                            broker_token_first_char: data.broker_token_first_char,
                        },
                    },
                }),
            };
            const response = await (0, request_1.makeRequestToDownstream)(req);
            const apiResponse = JSON.parse(response.body).data;
            if (!(apiResponse === null || apiResponse === void 0 ? void 0 : apiResponse.attributes)) {
                logger_1.log.trace({ apiResponse }, 'Unexpected Connection Allocation Server response');
                throw new Error('Unexpected connection allocation server response');
            }
            const snykRequestId = response.headers['snyk-request-id'] || '';
            logger_1.log.trace({ snykRequestId, apiResponse }, 'createConnection API response');
            const serverId = apiResponse.attributes.server_id;
            return serverId;
        }
        catch (err) {
            logger_1.log.trace({ err }, 'Error getting connection allocation.');
            throw new Error('Error getting connection allocation.');
        }
    }
}
exports.HttpDispatcherServiceClient = HttpDispatcherServiceClient;
//# sourceMappingURL=api.js.map