"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const uuid_1 = require("uuid");
const socket_1 = require("./socket");
const logger_1 = require("../logs/logger");
const forwardHttpRequest_1 = require("../common/relay/forwardHttpRequest");
const webserver_1 = require("../common/http/webserver");
const version_1 = __importDefault(require("../common/utils/version"));
const checks_handler_1 = require("./checks/api/checks-handler");
const healthcheckHandler_1 = require("./routesHandler/healthcheckHandler");
const systemCheckHandler_1 = require("./routesHandler/systemCheckHandler");
const processHooks_1 = require("./hooks/startup/processHooks");
process.on('uncaughtException', (error) => {
    if (error.message == 'read ECONNRESET') {
        logger_1.log.error({ msg: error.message, stackTrace: error.stack }, 'ECONNRESETs Catch all:', error.message);
    }
    else {
        logger_1.log.error({ msg: error.message, stackTrace: error.stack }, 'Uncaught exception:', error.message);
        process.exit(1);
    }
});
const main = async (clientOpts) => {
    var _a;
    try {
        logger_1.log.info({ version: version_1.default }, 'running in client mode');
        const brokerClientId = (0, uuid_1.v4)();
        logger_1.log.info({ brokerClientId }, 'generated broker client id');
        const hookResults = await (0, processHooks_1.processStartUpHooks)(clientOpts, brokerClientId);
        const identifyingMetadata = {
            capabilities: ['post-streams'],
            clientId: brokerClientId,
            filters: clientOpts.filters,
            preflightChecks: hookResults.preflightCheckResults,
            version: version_1.default,
        };
        const io = (0, socket_1.createWebSocket)(clientOpts, identifyingMetadata);
        // start the local webserver to listen for relay requests
        const { app, server } = (0, webserver_1.webserver)(clientOpts.config, clientOpts.port);
        // IMPORTANT: defined before relay (`app.all('/*', ...`)
        app.get('/health/checks', (0, checks_handler_1.handleChecksRoute)(clientOpts.config));
        app.get('/health/checks/:checkId', (0, checks_handler_1.handleCheckIdsRoutes)(clientOpts.config));
        app.get(clientOpts.config.brokerHealthcheckPath || '/healthcheck', (req, res, next) => {
            res.locals.io = io;
            next();
        }, healthcheckHandler_1.healthCheckHandler);
        app.get(clientOpts.config.brokerSystemcheckPath || '/systemcheck', (req, res, next) => {
            res.locals.clientOpts = clientOpts;
            next();
        }, systemCheckHandler_1.systemCheckHandler);
        // relay all other URL paths
        app.all('/*', (req, res, next) => {
            res.locals.io = io;
            next();
        }, (0, forwardHttpRequest_1.forwardHttpRequest)((_a = clientOpts.filters) === null || _a === void 0 ? void 0 : _a.public));
        return {
            io,
            close: (done) => {
                logger_1.log.info('client websocket is closing');
                server.close();
                io.destroy(function () {
                    logger_1.log.info('client websocket is closed');
                    if (done) {
                        return done();
                    }
                });
            },
        };
    }
    catch (err) {
        logger_1.log.warn({ err }, `Shutting down client`);
        throw err;
    }
};
exports.main = main;
//# sourceMappingURL=index.js.map