"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.systemCheckHandler = void 0;
const credentials_1 = require("../utils/credentials");
const logger_1 = require("../../logs/logger");
const systemCheckHandler = async (req, res) => {
    // Systemcheck is the broker client's ability to assert the network
    // reachability and some correctness of credentials for the service
    // being proxied by the broker client.
    const clientOpts = res.locals.clientOpts;
    const brokerClientValidationMethod = clientOpts.config.brokerClientValidationMethod || 'GET';
    const brokerClientValidationTimeoutMs = clientOpts.config.brokerClientValidationTimeoutMs || 5000;
    // set auth header according to config
    const { auths, rawCreds } = (0, credentials_1.loadCredentialsFromConfig)(clientOpts.config);
    // make the internal validation request
    const validationResults = [];
    let errorOccurred = true;
    if (auths.length > 0) {
        for (let i = 0; i < auths.length; i++) {
            logger_1.log.info(`Checking if credentials at index ${i} are valid`);
            const auth = auths[i];
            const rawCred = rawCreds[i];
            const { data, errorOccurred: err } = await (0, credentials_1.checkCredentials)(auth, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs);
            data['maskedCredentials'] =
                rawCred.length <= 6
                    ? '***'
                    : `${rawCred.substring(0, 3)}***${rawCred.substring(rawCred.length - 3)}`;
            validationResults.push(data);
            errorOccurred = err;
            logger_1.log.info('Credentials checked');
        }
    }
    else {
        logger_1.log.info('No credentials specified - checking if target can be accessed without credentials');
        const { data, errorOccurred: err } = await (0, credentials_1.checkCredentials)(null, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs);
        data['maskedCredentials'] = null;
        validationResults.push(data);
        errorOccurred = err;
    }
    if (errorOccurred) {
        return res.status(500).json(validationResults);
    }
    else {
        return res.status(200).json(validationResults);
    }
};
exports.systemCheckHandler = systemCheckHandler;
//# sourceMappingURL=systemCheckHandler.js.map