"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateForSymlinksInCreateTree = exports.convertBodyToGitHubTreePayload = exports.isGitHubCreateTreeEndpoint = void 0;
const logger_1 = require("../../../logs/logger");
const errors_1 = require("./errors");
function isGitHubCreateTreeEndpoint(options) {
    // https://docs.github.com/en/rest/git/trees?apiVersion=2022-11-28#create-a-tree
    return options.method === 'POST' && options.url.includes('/git/trees');
}
exports.isGitHubCreateTreeEndpoint = isGitHubCreateTreeEndpoint;
function convertBodyToGitHubTreePayload(body) {
    try {
        const bodyAsJson = JSON.parse(body);
        return {
            owner: bodyAsJson.owner,
            repo: bodyAsJson.repo,
            base_tree: bodyAsJson.base_tree,
            tree: bodyAsJson.tree,
        };
    }
    catch (error) {
        logger_1.log.error({ error }, 'Could not parse GitHub create tree payload body');
        throw new errors_1.GitHubTreeParsingError('Could not parse GitHub create tree payload body');
    }
}
exports.convertBodyToGitHubTreePayload = convertBodyToGitHubTreePayload;
function validateForSymlinksInCreateTree(createTree) {
    const treesWithSymlink = createTree.tree.filter((tree) => tree.mode === '120000' && tree.type === 'commit');
    if (treesWithSymlink.length > 0) {
        const paths = treesWithSymlink.map((tree) => tree.path).join(', ');
        throw new errors_1.GitHubTreeValidationError(`Symlinks are not allowed in GitHub tree payload: ${paths}`);
    }
}
exports.validateForSymlinksInCreateTree = validateForSymlinksInCreateTree;
//# sourceMappingURL=tree.js.map