"use strict";
// import '../common/http/patch-https-request-for-proxying';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebSocket = void 0;
const primus_1 = __importDefault(require("primus"));
const logger_1 = require("../logs/logger");
const primus_emitter_1 = __importDefault(require("primus-emitter"));
const reconnectHandler_1 = require("./socketHandlers/reconnectHandler");
const identifyHandler_1 = require("./socketHandlers/identifyHandler");
const errorHandler_1 = require("./socketHandlers/errorHandler");
const openHandler_1 = require("./socketHandlers/openHandler");
const closeHandler_1 = require("./socketHandlers/closeHandler");
const requestHandler_1 = require("./socketHandlers/requestHandler");
const chunkHandler_1 = require("./socketHandlers/chunkHandler");
const init_1 = require("./socketHandlers/init");
const createWebSocket = (clientOpts, identifyingMetadata) => {
    const Socket = primus_1.default.createSocket({
        transformer: 'engine.io',
        parser: 'EJSON',
        plugin: {
            emitter: primus_emitter_1.default,
        },
        pathname: `/primus/${clientOpts.config.brokerToken}`,
    });
    const urlWithServerId = new URL(clientOpts.config.brokerServerUrl);
    if (clientOpts.config.serverId) {
        urlWithServerId.searchParams.append('server_id', clientOpts.config.serverId);
    }
    clientOpts.config.brokerServerUrlForSocket = urlWithServerId.toString();
    // Will exponentially back-off from 0.5 seconds to a maximum of 20 minutes
    // Retry for a total period of around 4.5 hours
    const io = new Socket(clientOpts.config.brokerServerUrlForSocket, {
        reconnect: {
            factor: 1.5,
            retries: 30,
            max: 20 * 60 * 1000,
        },
        ping: parseInt(clientOpts.config.socketPingInterval) || 25000,
        pong: parseInt(clientOpts.config.socketPongTimeout) || 10000,
        timeout: parseInt(clientOpts.config.socketConnectTimeout) || 10000,
    });
    io.socketVersion = 1;
    io.socketType = 'client';
    logger_1.log.info({
        url: clientOpts.config.brokerServerUrlForSocket,
        serverId: clientOpts.config.serverId,
    }, 'broker client is connecting to broker server');
    (0, init_1.initializeSocketHandlers)(io, clientOpts);
    // Websocket events
    io.on('identify', (serverData) => (0, identifyHandler_1.identifyHandler)(serverData, io));
    io.on('reconnect scheduled', reconnectHandler_1.reconnectScheduledHandler);
    io.on('reconnect failed', () => (0, reconnectHandler_1.reconnectFailedHandler)(io));
    io.on('chunk', (0, chunkHandler_1.chunkHandler)(clientOpts));
    // prealably initialized
    io.on('request', (0, requestHandler_1.requestHandler)(clientOpts.config.brokerToken));
    io.on('error', errorHandler_1.errorHandler);
    io.on('open', () => (0, openHandler_1.openHandler)(io, clientOpts, identifyingMetadata));
    io.on('close', () => (0, closeHandler_1.closeHandler)(clientOpts));
    // only required if we're manually opening the connection
    // io.open();
    return io;
};
exports.createWebSocket = createWebSocket;
//# sourceMappingURL=socket.js.map