"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadBrokerConfig = exports.config = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const camelcase_1 = __importDefault(require("camelcase"));
const snyk_config_1 = require("snyk-config");
const dotenv_1 = __importDefault(require("dotenv"));
exports.config = {};
const loadBrokerConfig = () => {
    dotenv_1.default.config({
        path: path_1.default.join(process.cwd(), '.env'),
    });
    const localConfig = (0, snyk_config_1.loadConfig)(process.cwd());
    expand(process.env);
    exports.config = Object.assign({}, camelify(localConfig), camelify(process.env));
    if (exports.config.caCert) {
        exports.config.caCert = fs_1.default.readFileSync(path_1.default.resolve(process.cwd(), exports.config.caCert));
    }
    for (const [key, value] of Object.entries(exports.config)) {
        if ((key.endsWith('Pool') || key.endsWith('_POOL')) &&
            !Array.isArray(value)) {
            exports.config[key] = value.split(',').map((s) => s.trim());
        }
    }
};
exports.loadBrokerConfig = loadBrokerConfig;
function camelify(res) {
    return Object.keys(res).reduce((acc, key) => {
        const camelKey = (0, camelcase_1.default)(key);
        acc[camelKey] = res[key];
        acc[key] = res[key];
        return acc;
    }, {});
}
function expandValue(obj, value) {
    let poolFound = undefined;
    let keyWithPool = undefined;
    const variableRegex = /(\\?\$.+?\b)/g;
    const variableMatcher = value.match(variableRegex);
    if (variableMatcher) {
        for (const key of variableMatcher) {
            if (key[0] === '$' && obj[key.slice(1) + '_POOL']) {
                keyWithPool = key.slice(1);
                poolFound = key.slice(1) + '_POOL';
                break;
            }
        }
    }
    if (poolFound) {
        const values = [];
        let pool;
        if (Array.isArray(obj[poolFound])) {
            pool = obj[poolFound];
        }
        else {
            pool = obj[poolFound].split(',').map((s) => s.trim());
            obj[poolFound] = pool;
        }
        for (const o of pool) {
            values.push(value.replace(variableRegex, (all, key) => {
                if (key[0] === '$') {
                    const keyToReplace = key.slice(1);
                    return keyToReplace === keyWithPool ? o : obj[keyToReplace] || key;
                }
                return key;
            }));
        }
        return values;
    }
    else {
        return value.replace(variableRegex, (all, key) => {
            if (key[0] === '$') {
                const keyToReplace = key.slice(1);
                return obj[keyToReplace] || key;
            }
            return key;
        });
    }
}
function expand(obj) {
    const keys = Object.keys(obj);
    for (const key of keys) {
        const value = expandValue(obj, obj[key]);
        if (value && Array.isArray(value)) {
            obj[key + '_POOL'] = value;
        }
        else if (value !== obj[key]) {
            obj[key] = value;
        }
    }
    return obj;
}
//# sourceMappingURL=config.js.map