"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
var _BrokerServerPostResponseHandler_instances, _BrokerServerPostResponseHandler_buffer, _BrokerServerPostResponseHandler_brokerTransformer, _BrokerServerPostResponseHandler_logContext, _BrokerServerPostResponseHandler_config, _BrokerServerPostResponseHandler_brokerToken, _BrokerServerPostResponseHandler_streamingId, _BrokerServerPostResponseHandler_serverId, _BrokerServerPostResponseHandler_requestId, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, _BrokerServerPostResponseHandler_initHttpClientRequest, _BrokerServerPostResponseHandler_sendIoData, _BrokerServerPostResponseHandler_handleRequestError;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrokerServerPostResponseHandler = void 0;
const logger_1 = require("../../logs/logger");
const stream_1 = __importDefault(require("stream"));
const promises_1 = require("stream/promises");
const replace_vars_1 = require("../utils/replace-vars");
const version_1 = __importDefault(require("../utils/version"));
const proxy_from_env_1 = require("proxy-from-env");
const global_agent_1 = require("global-agent");
const https_1 = __importDefault(require("https"));
const http_1 = __importDefault(require("http"));
const config_1 = require("../config");
const BROKER_CONTENT_TYPE = 'application/vnd.broker.stream+octet-stream';
const client = ((_a = config_1.config.brokerServerUrl) === null || _a === void 0 ? void 0 : _a.startsWith('https')) ? https_1.default : http_1.default;
const agent = new client.Agent({
    keepAlive: true,
    keepAliveMsecs: 600000,
});
if (process.env.HTTP_PROXY || process.env.http_proxy) {
    process.env.HTTP_PROXY = process.env.HTTP_PROXY || process.env.http_proxy;
}
if (process.env.HTTPS_PROXY || process.env.https_proxy) {
    process.env.HTTPS_PROXY = process.env.HTTPS_PROXY || process.env.https_proxy;
}
if (process.env.NP_PROXY || process.env.no_proxy) {
    process.env.NO_PROXY = process.env.NO_PROXY || process.env.no_proxy;
}
const proxyUri = (0, proxy_from_env_1.getProxyForUrl)(config_1.config.brokerServerUrl);
if (proxyUri) {
    (0, global_agent_1.bootstrap)({
        environmentVariableNamespace: '',
    });
}
class BrokerServerPostResponseHandler {
    constructor(logContext, config, brokerToken, serverId, requestId) {
        _BrokerServerPostResponseHandler_instances.add(this);
        _BrokerServerPostResponseHandler_buffer.set(this, void 0);
        _BrokerServerPostResponseHandler_brokerTransformer.set(this, void 0);
        _BrokerServerPostResponseHandler_logContext.set(this, void 0);
        _BrokerServerPostResponseHandler_config.set(this, void 0);
        _BrokerServerPostResponseHandler_brokerToken.set(this, void 0);
        _BrokerServerPostResponseHandler_streamingId.set(this, void 0);
        _BrokerServerPostResponseHandler_serverId.set(this, void 0);
        _BrokerServerPostResponseHandler_requestId.set(this, void 0);
        _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler.set(this, void 0);
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_logContext, logContext, "f");
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_config, config, "f");
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_brokerToken, brokerToken, "f");
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_serverId, serverId, "f");
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_requestId, requestId, "f");
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_buffer, new stream_1.default.PassThrough({ highWaterMark: 1048576 }), "f");
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").on('error', (e) => logger_1.log.error({
            ...__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"),
            error: e,
            stackTrace: new Error('stacktrace generator').stack,
        }, 'received error sending data to broker server post request buffer'));
    }
    async forwardRequest(response, streamingID) {
        try {
            __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_streamingId, streamingID, "f");
            let prevPartialChunk;
            response.socket.on('error', (err) => {
                logger_1.log.error({
                    msg: err.message,
                    err: err,
                    stackTrace: new Error('stacktrace generator').stack,
                }, 'Socket Response error in Streaming from downstream');
            });
            const status = (response === null || response === void 0 ? void 0 : response.statusCode) || 500;
            logger_1.log.debug({
                ...__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"),
                responseStatus: status,
                responseHeaders: response.headers,
            }, 'response received, setting up stream to Broker Server');
            const isResponseJson = isJson(response.headers);
            const ioData = JSON.stringify({
                status,
                headers: response.headers,
            });
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_initHttpClientRequest).call(this);
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_sendIoData).call(this, ioData);
            logger_1.log.debug(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), 'successfully sent status & headers to Broker Server');
            const localLogContext = __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f");
            // TODO: break into 2 distinct transform, only to add to pipeline if conditions are met
            // Take out of here if possible
            logger_1.log.debug(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), 'Setting up transformers');
            __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_brokerTransformer, new stream_1.default.Transform({
                transform(chunk, encoding, callback) {
                    const httpBody = config_1.config && config_1.config.LOG_ENABLE_BODY === 'true'
                        ? { body: chunk.toString() }.body
                        : null;
                    logger_1.log.debug({ ...localLogContext, chunkLength: chunk.length, httpBody }, 'writing data to buffer');
                    if (config_1.config.RES_BODY_URL_SUB && isResponseJson) {
                        const { newChunk, partial } = (0, replace_vars_1.replaceUrlPartialChunk)(Buffer.from(chunk).toString(), prevPartialChunk, config_1.config);
                        prevPartialChunk = partial;
                        chunk = newChunk;
                    }
                    callback(null, chunk);
                },
            }), "f");
            response.on('error', __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_handleRequestError).call(this));
            if ((config_1.config && config_1.config.LOG_ENABLE_BODY === 'true') ||
                (config_1.config.RES_BODY_URL_SUB && isResponseJson)) {
                logger_1.log.debug(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), 'Pipelining with body logging on or Body replace ');
                await (0, promises_1.pipeline)(response, __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f"), __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerTransformer, "f"), __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, "f"));
            }
            else {
                logger_1.log.debug(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), 'Pipelining standard');
                await (0, promises_1.pipeline)(response, __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f"), __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, "f"));
            }
        }
        catch (err) {
            logger_1.log.error({ err }, 'Error in forwarding the request to broker server pipeline');
        }
    }
    async sendData(responseData, streamingID) {
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_streamingId, streamingID, "f");
        const body = responseData.body;
        delete responseData.body;
        logger_1.log.debug({ ...__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), responseData, body }, 'posting internal response back to Broker Server as it is expecting streaming response');
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_initHttpClientRequest).call(this);
        (0, promises_1.pipeline)(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f"), __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, "f"));
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_sendIoData).call(this, JSON.stringify(responseData));
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").write(JSON.stringify(body));
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").end();
    }
}
exports.BrokerServerPostResponseHandler = BrokerServerPostResponseHandler;
_BrokerServerPostResponseHandler_buffer = new WeakMap(), _BrokerServerPostResponseHandler_brokerTransformer = new WeakMap(), _BrokerServerPostResponseHandler_logContext = new WeakMap(), _BrokerServerPostResponseHandler_config = new WeakMap(), _BrokerServerPostResponseHandler_brokerToken = new WeakMap(), _BrokerServerPostResponseHandler_streamingId = new WeakMap(), _BrokerServerPostResponseHandler_serverId = new WeakMap(), _BrokerServerPostResponseHandler_requestId = new WeakMap(), _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler = new WeakMap(), _BrokerServerPostResponseHandler_instances = new WeakSet(), _BrokerServerPostResponseHandler_initHttpClientRequest = async function _BrokerServerPostResponseHandler_initHttpClientRequest() {
    try {
        const url = new URL(`${__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_config, "f").brokerServerUrl}/response-data/${__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerToken, "f")}/${__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_streamingId, "f")}`);
        if (__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_serverId, "f")) {
            url.searchParams.append('server_id', __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_serverId, "f"));
        }
        const brokerServerPostRequestUrl = url.toString();
        const options = {
            method: 'post',
            headers: {
                'Snyk-Request-Id': `${__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_requestId, "f")}`,
                'Content-Type': BROKER_CONTENT_TYPE,
                Connection: 'Keep-Alive',
                'Keep-Alive': 'timeout=600000, max=1000000',
                'user-agent': 'Snyk Broker client ' + version_1.default,
            },
            agent: agent,
            timeout: 600000,
        };
        __classPrivateFieldSet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, client.request(brokerServerPostRequestUrl, options), "f");
        __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, "f")
            .on('error', (e) => {
            logger_1.log.error({
                error: e,
                stackTrace: new Error('stacktrace generator').stack,
            }, 'received error sending data via POST to Broker Server');
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").destroy(e);
        })
            .on('response', (r) => {
            var _a;
            r.socket.on('error', (err) => {
                logger_1.log.error({
                    msg: err.message,
                    err: err,
                    stackTrace: new Error('stacktrace generator').stack,
                }, 'Stream Socket Response error in POST to Broker Server');
            });
            r.on('error', (err) => {
                logger_1.log.error({
                    msg: err.message,
                    err: err,
                    stackTrace: new Error('stacktrace generator').stack,
                }, 'Stream Response error in POST to Broker Server');
            });
            if (r.statusCode !== 200) {
                logger_1.log.error({
                    statusCode: r.statusCode,
                    statusMessage: r.statusMessage,
                    headers: r.headers,
                    body: (_a = r.body) === null || _a === void 0 ? void 0 : _a.toString(),
                    stackTrace: new Error('stacktrace generator').stack,
                }, 'Received unexpected HTTP response POSTing data to Broker Server');
            }
        })
            .on('finish', () => {
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_brokerSrvPostRequestHandler, "f").removeAllListeners();
        });
        logger_1.log.debug(__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), 'POST Request Client setup');
    }
    catch (err) {
        logger_1.log.error({ err }, 'Error init Client for POST Broker Server Stream');
    }
}, _BrokerServerPostResponseHandler_sendIoData = function _BrokerServerPostResponseHandler_sendIoData(ioData) {
    const ioDataLength = ioData.length;
    logger_1.log.debug({ ...__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"), ioDataLength }, `sending ioData (Status & Headers) to Broker Server`);
    // Would be nice if there were a Unit32Array or a writeUint32 method, but noooooo...
    const ioDataLengthBinary = new Uint8Array(4);
    ioDataLengthBinary[0] = ioDataLength & 0xff;
    ioDataLengthBinary[1] = (ioDataLength >> 8) & 0xff;
    ioDataLengthBinary[2] = (ioDataLength >> 16) & 0xff;
    ioDataLengthBinary[3] = (ioDataLength >> 24) & 0xff;
    __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").cork();
    __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").write(ioDataLengthBinary);
    __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").write(ioData);
    __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").uncork();
}, _BrokerServerPostResponseHandler_handleRequestError = function _BrokerServerPostResponseHandler_handleRequestError() {
    // For reasons unknown, doing foo.on(this.#func)
    // doesn't work - you need return a function from here
    return (error) => {
        logger_1.log.error({
            ...__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_logContext, "f"),
            error,
            stackTrace: new Error('stacktrace generator').stack,
        }, 'received error from request while piping to Broker Server');
        // If we already have a buffer object, then we've already started sending data back to the original requestor,
        // so we have to destroy the stream and let that flow through the system
        // If we *don't* have a buffer object, then there was a major failure with the request (e.g., host not found), so
        // we will forward that directly to the Broker Server
        if (__classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f")) {
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").destroy(error);
        }
        else {
            const body = JSON.stringify({ error: error });
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_instances, "m", _BrokerServerPostResponseHandler_sendIoData).call(this, JSON.stringify({
                status: 500,
                headers: {
                    'Content-Length': `${body.length}`,
                    'Content-Type': 'application/json',
                },
            }));
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").write(body);
            __classPrivateFieldGet(this, _BrokerServerPostResponseHandler_buffer, "f").end();
        }
    };
};
function isJson(responseHeaders) {
    var _a;
    return ((_a = responseHeaders['content-type']) === null || _a === void 0 ? void 0 : _a.includes('json')) || false;
}
//# sourceMappingURL=downstream-post-stream-to-server.js.map