"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StreamResponseHandler = exports.streamsStore = void 0;
const metrics_1 = require("../utils/metrics");
const node_cache_1 = __importDefault(require("node-cache"));
const config_1 = require("../config");
exports.streamsStore = new node_cache_1.default({
    stdTTL: parseInt(config_1.config.cacheExpiry) || 3600,
    checkperiod: parseInt(config_1.config.cacheCheckPeriod) || 60,
    useClones: false,
});
class StreamResponseHandler {
    // streamBuffer;
    // response;
    // streamSize = 0;
    static create(streamingID) {
        const stream = exports.streamsStore.get(streamingID);
        if (!stream) {
            return null;
        }
        const streamResponse = stream;
        return new StreamResponseHandler(streamingID, streamResponse.streamBuffer, streamResponse.response);
    }
    constructor(streamingID, streamBuffer, response) {
        this.writeStatusAndHeaders = (statusAndHeaders) => {
            this.streamResponse.response
                .status(statusAndHeaders.status)
                .set(statusAndHeaders.headers);
        };
        this.writeChunk = (chunk, waitForDrainCb) => {
            this.streamResponse.streamSize += chunk.length;
            if (!this.streamResponse.streamBuffer.write(chunk) && waitForDrainCb) {
                waitForDrainCb(this.streamResponse.streamBuffer);
            }
        };
        this.finished = () => {
            this.streamResponse.streamBuffer.end();
            exports.streamsStore.del(this.streamingID);
            (0, metrics_1.observeResponseSize)({
                bytes: this.streamResponse.streamSize,
                isStreaming: true,
            });
        };
        this.destroy = (error) => {
            this.streamResponse.streamBuffer.destroy(error);
            exports.streamsStore.del(this.streamingID);
        };
        this.streamingID = streamingID;
        this.streamResponse = { streamBuffer, response, streamSize: 0 };
    }
}
exports.StreamResponseHandler = StreamResponseHandler;
//# sourceMappingURL=server-post-stream-handler.js.map