"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.incrementWebSocketRequestsTotal = exports.incrementHttpRequestsTotal = exports.incrementUnableToSizeResponse = exports.observeResponseSize = exports.decrementSocketConnectionGauge = exports.incrementSocketConnectionGauge = void 0;
const prom_client_1 = require("prom-client");
const socketConnectionGauge = new prom_client_1.Gauge({
    name: 'broker_socket_connection_total',
    help: 'Number of active socket connections',
});
const responseSizeHistogram = new prom_client_1.Histogram({
    name: 'broker_response_size_bytes',
    help: 'The size of broker server responses in bytes',
    labelNames: ['isStreaming'],
    buckets: [
        1024,
        5120,
        10240,
        25600,
        51200,
        102400,
        512000,
        1048576,
        10485760,
        20971520, // 20mb 'maxLength' in socket.js
    ],
});
const unableToSizeResponseCounter = new prom_client_1.Counter({
    name: 'broker_unable_to_size_response_count',
    help: 'A count of the number of times broker server was unable to size a response',
});
const httpRequestsTotal = new prom_client_1.Counter({
    name: 'http_request_total',
    help: 'Number of HTTP requests',
    labelNames: ['rejectedByFilter', 'flowDirection'],
});
const webSocketRequestsTotal = new prom_client_1.Counter({
    name: 'broker_ws_request_total',
    help: 'Number of requests received via WebSocket',
    labelNames: ['rejectedByFilter', 'flowDirection'],
});
function incrementSocketConnectionGauge() {
    socketConnectionGauge.inc(1);
}
exports.incrementSocketConnectionGauge = incrementSocketConnectionGauge;
function decrementSocketConnectionGauge() {
    socketConnectionGauge.dec(1);
}
exports.decrementSocketConnectionGauge = decrementSocketConnectionGauge;
function observeResponseSize({ bytes, isStreaming }) {
    responseSizeHistogram.observe({ isStreaming }, bytes);
}
exports.observeResponseSize = observeResponseSize;
function incrementUnableToSizeResponse() {
    unableToSizeResponseCounter.inc(1);
}
exports.incrementUnableToSizeResponse = incrementUnableToSizeResponse;
function incrementHttpRequestsTotal(rejectedByFilter, flowDirection) {
    httpRequestsTotal.inc({ rejectedByFilter: rejectedByFilter, flowDirection: flowDirection }, 1);
}
exports.incrementHttpRequestsTotal = incrementHttpRequestsTotal;
function incrementWebSocketRequestsTotal(rejectedByFilter, flowDirection) {
    webSocketRequestsTotal.inc({ rejectedByFilter: rejectedByFilter, flowDirection: flowDirection }, 1);
}
exports.incrementWebSocketRequestsTotal = incrementWebSocketRequestsTotal;
//# sourceMappingURL=metrics.js.map