"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const logger_1 = require("../logs/logger");
const socket_1 = require("./socket");
const version_1 = __importDefault(require("../common/utils/version"));
const prometheus_middleware_1 = require("./utils/prometheus-middleware");
const broker_middleware_1 = require("./broker-middleware");
const webserver_1 = require("../common/http/webserver");
const dispatcher_1 = require("./infra/dispatcher");
const postResponseHandler_1 = require("./routesHandlers/postResponseHandler");
const connectionStatusHandler_1 = require("./routesHandlers/connectionStatusHandler");
const httpRequestHandler_1 = require("./routesHandlers/httpRequestHandler");
const initHandlers_1 = require("./socketHandlers/initHandlers");
const main = async (serverOpts) => {
    logger_1.log.info({ version: version_1.default }, 'running in server mode');
    // start the local webserver to listen for relay requests
    const { app, server } = (0, webserver_1.webserver)(serverOpts.config, serverOpts.port);
    // Requires are done recursively, so this is here to avoid contaminating the Client
    const onSignal = async () => {
        logger_1.log.debug('received exit signal, closing server');
        await (0, dispatcher_1.serverStopping)(() => {
            process.exit(0);
        });
    };
    process.once('SIGINT', onSignal);
    process.once('SIGTERM', onSignal);
    await (0, dispatcher_1.serverStarting)();
    // bind the socket server to the web server
    const { io } = (0, socket_1.bindSocketToWebserver)(server, serverOpts);
    if (!process.env.JEST_WORKER_ID) {
        app.use((0, prometheus_middleware_1.applyPrometheusMiddleware)());
    }
    app.get('/connection-status/:token', connectionStatusHandler_1.connectionStatusHandler);
    app.all('/broker/:token/*', httpRequestHandler_1.overloadHttpRequestWithConnectionDetailsMiddleware, broker_middleware_1.validateBrokerTypeMiddleware, (0, initHandlers_1.getForwardHttpRequestHandler)());
    app.post('/response-data/:brokerToken/:streamingId', postResponseHandler_1.handlePostResponse);
    app.get('/', (req, res) => res.status(200).json({ ok: true, version: version_1.default }));
    app.get('/healthcheck', (req, res) => res.status(200).json({ ok: true, version: version_1.default }));
    return {
        io,
        close: (done) => {
            logger_1.log.info('server websocket is closing');
            server.close();
            io.destroy(function () {
                logger_1.log.info('server websocket is closed');
                if (done) {
                    return done();
                }
            });
        },
    };
};
exports.main = main;
//# sourceMappingURL=index.js.map