"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _a, _b;
var _DispatcherClient_instances, _DispatcherClient_url, _DispatcherClient_hostname, _DispatcherClient_id, _DispatcherClient_version, _DispatcherClient_makeRequest;
Object.defineProperty(exports, "__esModule", { value: true });
exports.serverStopping = exports.serverStarting = exports.clientDisconnected = exports.clientPinged = exports.clientConnected = void 0;
const token_1 = require("../../common/utils/token");
const logger_1 = require("../../logs/logger");
const config_1 = require("../../common/config");
const axios_1 = require("../../common/http/axios");
const uuid_1 = require("uuid");
class DispatcherClient {
    constructor(url, hostname, id, version) {
        _DispatcherClient_instances.add(this);
        _DispatcherClient_url.set(this, void 0);
        _DispatcherClient_hostname.set(this, void 0);
        _DispatcherClient_id.set(this, void 0);
        _DispatcherClient_version.set(this, void 0);
        __classPrivateFieldSet(this, _DispatcherClient_url, url, "f");
        __classPrivateFieldSet(this, _DispatcherClient_hostname, hostname, "f");
        __classPrivateFieldSet(this, _DispatcherClient_id, id || 0, "f");
        __classPrivateFieldSet(this, _DispatcherClient_version, version || '2022-12-02~experimental', "f");
    }
    async serverStarting() {
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { requestType: 'server-starting' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}`, 'post', {
            data: {
                attributes: {
                    health_check_link: `http://${__classPrivateFieldGet(this, _DispatcherClient_hostname, "f")}/healthcheck`,
                },
            },
        });
    }
    async serverStopping(cb) {
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { requestType: 'server-stopping' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}`, 'delete', cb);
    }
    async clientConnected(token, clientId, clientVersion, requestType = 'client-connected', time = -1) {
        const hashedToken = (0, token_1.hashToken)(token);
        const url = new URL(`${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}/connections/${hashedToken}`);
        if (clientId) {
            url.searchParams.append('broker_client_id', clientId);
        }
        if (time != -1) {
            url.searchParams.append('latency', `${Date.now() - time}`);
        }
        url.searchParams.append('request_type', requestType);
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { hashedToken, clientId, requestType: requestType }, url.toString(), 'post', {
            data: {
                attributes: {
                    health_check_link: `http://${__classPrivateFieldGet(this, _DispatcherClient_hostname, "f")}/healthcheck`,
                    broker_client_version: `${clientVersion}`,
                },
            },
        });
    }
    async clientDisconnected(token, clientId) {
        const hashedToken = (0, token_1.hashToken)(token);
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { hashedToken, clientId, requestType: 'client-disconnected' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}/connections/${hashedToken}${clientId ? `?broker_client_id=${clientId}` : ''}`, 'delete');
    }
}
_DispatcherClient_url = new WeakMap(), _DispatcherClient_hostname = new WeakMap(), _DispatcherClient_id = new WeakMap(), _DispatcherClient_version = new WeakMap(), _DispatcherClient_instances = new WeakSet(), _DispatcherClient_makeRequest = async function _DispatcherClient_makeRequest(logContext, url, method, requestBody, cb) {
    const requestId = (0, uuid_1.v4)();
    // version *must* be provided
    const urlWithVersion = new URL(url);
    urlWithVersion.searchParams.append('version', __classPrivateFieldGet(this, _DispatcherClient_version, "f"));
    url = urlWithVersion.toString();
    try {
        const response = await axios_1.axiosInstance.request({
            url,
            method,
            data: requestBody && JSON.stringify(requestBody),
            headers: {
                'Content-Type': 'application/vnd.api+json',
                Connection: 'Keep-Alive',
                'Keep-Alive': 'timeout=60, max=10',
                'Snyk-Request-Id': requestId,
            },
        });
        const statusCode = response.status;
        const headers = response.headers;
        const body = response.data;
        if (statusCode >= 300) {
            logger_1.log.error({
                ...logContext,
                requestId,
                statusCode,
                headers,
                body,
                dispatcherUrl: __classPrivateFieldGet(this, _DispatcherClient_url, "f"),
                dispatcherVersion: __classPrivateFieldGet(this, _DispatcherClient_version, "f"),
                serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"),
            }, 'received unexpected status code communicating with Dispatcher');
        }
        else {
            logger_1.log.trace({ ...logContext, serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"), requestId }, 'successfully sent request to Dispatcher');
        }
    }
    catch (e) {
        logger_1.log.error({
            ...logContext,
            requestId,
            errorMessage: e.message,
            stackTrace: new Error('stack generator').stack,
            dispatcherUrl: __classPrivateFieldGet(this, _DispatcherClient_url, "f"),
            dispatcherVersion: __classPrivateFieldGet(this, _DispatcherClient_version, "f"),
            serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"),
        }, 'received error communicating with Dispatcher');
    }
    if (cb)
        cb();
};
if (config_1.config.dispatcherUrl) {
    const kc = new DispatcherClient(config_1.config.dispatcherUrl, config_1.config.hostname, (_a = config_1.config.hostname) === null || _a === void 0 ? void 0 : _a.substring(((_b = config_1.config.hostname) === null || _b === void 0 ? void 0 : _b.lastIndexOf('-')) + 1), config_1.config.dispatcherVersion);
    exports.clientConnected = async function (token, clientId, clientVersion) {
        return kc.clientConnected(token, clientId, clientVersion);
    };
    exports.clientPinged = async function (token, clientId, clientVersion, time) {
        return kc.clientConnected(token, clientId, clientVersion, 'client-pinged', time);
    };
    exports.clientDisconnected = async function (token, clientId) {
        return kc.clientDisconnected(token, clientId);
    };
    exports.serverStarting = async function () {
        return kc.serverStarting();
    };
    exports.serverStopping = async function (cb) {
        return kc.serverStopping(cb);
    };
}
else {
    logger_1.log.error('DISPATCHER_URL not set - creating no-op functions to ensure Server still functions');
    exports.clientConnected = async function () {
        logger_1.log.trace('client connected - no-op instead of notifying dispatcher');
    };
    exports.clientPinged = async function () {
        logger_1.log.trace('client pinged - no-op instead of notifying dispatcher');
    };
    exports.clientDisconnected = async function () {
        logger_1.log.trace('client disconnected - no-op instead of notifying dispatcher');
    };
    exports.serverStarting = async function () {
        logger_1.log.info('server started - no-op instead of notifying dispatcher');
    };
    exports.serverStopping = async function (cb) {
        logger_1.log.info('server stopping - no-op instead of notifying dispatcher');
        cb();
    };
}
//# sourceMappingURL=dispatcher.js.map