"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bindSocketToWebserver = exports.getSocketConnections = void 0;
const primus_1 = __importDefault(require("primus"));
const primus_emitter_1 = __importDefault(require("primus-emitter"));
const logger_1 = require("../logs/logger");
const errorHandler_1 = require("./socketHandlers/errorHandler");
const connectionHandler_1 = require("./socketHandlers/connectionHandler");
const initHandlers_1 = require("./socketHandlers/initHandlers");
const socketConnections = new Map();
const getSocketConnections = () => {
    return socketConnections;
};
exports.getSocketConnections = getSocketConnections;
const socket = ({ server, serverOpts: ServerOpts }) => {
    const ioConfig = {
        transformer: 'engine.io',
        parser: 'EJSON',
        maxLength: parseInt(ServerOpts.config.socketMaxResponseLength) || 22020096,
        transport: {
            allowEIO3: true,
            pingInterval: parseInt(ServerOpts.config.socketPingInterval) || 25000,
            pingTimeout: parseInt(ServerOpts.config.socketPingTimeout) || 20000,
        },
        compression: Boolean(ServerOpts.config.socketUseCompression) || false,
    };
    const io = new primus_1.default(server, ioConfig);
    io.socketType = 'server';
    io.socketVersion = 1;
    io.plugin('emitter', primus_emitter_1.default);
    logger_1.log.info(ioConfig, 'using io config');
    (0, initHandlers_1.initConnectionHandler)(ServerOpts, io);
    io.on('error', errorHandler_1.handleIoError);
    io.on('connection', connectionHandler_1.handleSocketConnection);
    return { io };
};
const bindSocketToWebserver = (server, serverOpts) => {
    // bind the socket server to the web server
    return socket({
        server,
        serverOpts,
    });
};
exports.bindSocketToWebserver = bindSocketToWebserver;
//# sourceMappingURL=socket.js.map