"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleConnectionCloseOnSocket = void 0;
const metrics_1 = require("../../common/utils/metrics");
const logger_1 = require("../../logs/logger");
const dispatcher_1 = require("../infra/dispatcher");
const socket_1 = require("../socket");
const token_1 = require("../utils/token");
const handleConnectionCloseOnSocket = (closeReason, socket, token, clientId, identified) => {
    if (token) {
        const { maskedToken, hashedToken } = (0, token_1.getDesensitizedToken)(token);
        if (identified) {
            const connections = (0, socket_1.getSocketConnections)();
            const clientPool = connections
                .get(token)
                .filter((_) => _.socket !== socket);
            const filteredClientPool = (clientPool === null || clientPool === void 0 ? void 0 : clientPool.filter((_) => _.socket !== socket)) || '';
            logger_1.log.info({
                closeReason,
                maskedToken,
                hashedToken,
                remainingConnectionsCount: clientPool.length,
            }, 'client connection closed');
            if (filteredClientPool.length) {
                connections.set(token, filteredClientPool);
            }
            else {
                logger_1.log.info({ maskedToken, hashedToken }, 'removing client');
                connections.delete(token);
            }
            (0, metrics_1.decrementSocketConnectionGauge)();
        }
        else {
            logger_1.log.warn({ maskedToken, hashedToken }, 'client disconnected before identifying itself');
        }
        setImmediate(async () => await (0, dispatcher_1.clientDisconnected)(token, clientId));
    }
};
exports.handleConnectionCloseOnSocket = handleConnectionCloseOnSocket;
//# sourceMappingURL=closeHandler.js.map