"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleSocketConnection = void 0;
const logger_1 = require("../../logs/logger");
const token_1 = require("../utils/token");
const identifyHandler_1 = require("./identifyHandler");
const closeHandler_1 = require("./closeHandler");
const errorHandler_1 = require("./errorHandler");
const handleSocketConnection = (socket) => {
    let clientId = null;
    let identified = false;
    const token = socket.request.uri.pathname
        .replaceAll(/\/primus\/([^/]+)\//g, '$1')
        .toLowerCase();
    const desensitizedToken = (0, token_1.getDesensitizedToken)(token);
    logger_1.log.info({ desensitizedToken }, 'new client connection');
    socket.send('identify', { capabilities: ['receive-post-streams'] });
    // TODO: type clientData and make sure we get the version
    socket.on('identify', (clientData) => {
        clientId = clientData.metadata.clientId;
        identified = (0, identifyHandler_1.handleIdentifyOnSocket)(clientData, socket, token);
    });
    ['close', 'end', 'disconnect'].forEach((e) => socket.on(e, () => (0, closeHandler_1.handleConnectionCloseOnSocket)(e, socket, token, clientId, identified)));
    socket.on('error', (error) => (0, errorHandler_1.handleSocketError)(error));
};
exports.handleSocketConnection = handleSocketConnection;
//# sourceMappingURL=connectionHandler.js.map