"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleIdentifyOnSocket = exports.initIdentifyHandler = void 0;
const LegacyStreamResponseHandler_1 = require("../../common/relay/LegacyStreamResponseHandler");
const metrics_1 = require("../../common/utils/metrics");
const logger_1 = require("../../logs/logger");
const dispatcher_1 = require("../infra/dispatcher");
const socket_1 = require("../socket");
const socket_2 = require("../utils/socket");
const token_1 = require("../utils/token");
const initHandlers_1 = require("./initHandlers");
let response;
const streamingResponse = LegacyStreamResponseHandler_1.legacyStreamResponseHandler;
const initIdentifyHandler = () => {
    response = (0, initHandlers_1.getForwardWebSocketRequestHandler)();
};
exports.initIdentifyHandler = initIdentifyHandler;
// TODO decide if the socket doesn't identify itself within X period,
// should we toss it away?
const handleIdentifyOnSocket = (clientData, socket, token) => {
    // clientData can be a string token coming from older broker clients,
    // OR an object coming from newer clients in the form of { token, metadata }
    if (typeof clientData === 'object') {
        token = clientData.token && clientData.token.toLowerCase();
    }
    else {
        token = clientData.toLowerCase(); // lowercase to standardise tokens
        // stub a proper clientData, signal client is too old
        clientData = { token, metadata: { version: 'pre-4.27' } };
    }
    if (!token) {
        logger_1.log.warn({ token, metadata: (0, socket_2.metadataWithoutFilters)(clientData.metadata) }, 'new client connection identified without a token');
        return false;
    }
    const { maskedToken, hashedToken } = (0, token_1.getDesensitizedToken)(token);
    const clientId = clientData.metadata.clientId;
    const clientVersion = clientData.metadata.version;
    // TODO: If version < cutoff version, then alert first, then deny
    //   if(clientVersion < minimalVersion) {
    //     socket.send('error', { message: `Broker Client Version is outdated. Minimal version: ${minimalVersion}. Please upgrade to latest version` });
    //   }
    //   if(clientVersion < minimalSupportedVersion) {
    //     socket.send('warning', { message: `Broker Client Version is outdated. Minimal version: ${minimalVersion}. Please upgrade to latest version` });
    //   }
    logger_1.log.info({
        maskedToken,
        hashedToken,
        metadata: (0, socket_2.metadataWithoutFilters)(clientData.metadata),
    }, 'new client connection identified');
    const connections = (0, socket_1.getSocketConnections)();
    const clientPool = connections.get(token) || [];
    clientPool.unshift({
        socket,
        socketType: 'server',
        socketVersion: 1,
        metadata: clientData.metadata,
    });
    connections.set(token, clientPool);
    socket.on('chunk', streamingResponse(token));
    socket.on('request', response(token));
    socket.on('incoming::ping', (time) => {
        setImmediate(async () => await (0, dispatcher_1.clientPinged)(token, clientId, clientVersion, time));
    });
    setImmediate(async () => await (0, dispatcher_1.clientConnected)(token, clientId, clientVersion));
    (0, metrics_1.incrementSocketConnectionGauge)();
    return true;
};
exports.handleIdentifyOnSocket = handleIdentifyOnSocket;
//# sourceMappingURL=identifyHandler.js.map